<?php 
class WPJAM_Posts_List_Table extends WPJAM_List_Table{
	private $post_type	= '';

	public function __construct($args = []){
		$current_screen	= get_current_screen();

		$screen_id	= $current_screen->id;
		$post_type	= $screen_id == 'upload' ? 'attachment' : $current_screen->post_type;
		$pt_obj		= get_post_type_object($post_type);

		$this->post_type	= $post_type;
		$args['title']		= $pt_obj->label;
		$args['capability']	= 'edit_post';
		$args['data_type']	= 'post_meta';
		$args['actions']	= wpjam_sort_items(WPJAM_List_Table_Action::get_by_screen_id($screen_id));
		$args['actions']	= apply_filters_deprecated('wpjam_'.$post_type.'_posts_actions', [$args['actions'], $post_type], 'WPJAM Basic 4.6');

		if(isset($args['actions']['add']) && empty($args['actions']['add']['capability'])){
			$args['actions']['add']['capability']	= $pt_obj->cap->create_posts;
		}

		$this->_args	= $this->parse_args($args);

		if(wp_doing_ajax()){
			add_action('wp_ajax_wpjam-list-table-action',	[$this, 'ajax_response']);
		}else{
			add_action('admin_head',	[$this, 'admin_head']);
			add_action('admin_footer',	[$this, '_js_vars']);
		}

		if(!wp_doing_ajax() || (wp_doing_ajax() && $_POST['action']=='inline-save')){
			add_action('wpjam_html',	[$this, 'filter_html']);
		}

		if(wp_doing_ajax() && $_POST['action'] == 'wpjam-list-table-action' && $_POST['list_action_type'] != 'form'){
			add_filter('wpjam_ajax_response',	[$this, 'filter_ajax_response']);
		}

		add_filter('request',		[$this, 'filter_request']);
		add_action('pre_get_posts',	[$this, 'pre_get_posts']);

		add_filter('bulk_actions-'.$screen_id,	[$this, 'filter_bulk_actions']);
		add_action('restrict_manage_posts',		[$this, 'add_taxonomy_dropdown'], 1);

		if($post_type == 'attachment'){
			add_filter('media_row_actions',	[$this, 'filter_row_actions'],1,2);

			add_filter('manage_media_columns',			[$this, 'filter_columns']);
			add_filter('manage_media_custom_column',	[$this, 'filter_custom_column'], 10, 2);
		}else{
			add_filter('map_meta_cap',	[$this, 'filter_map_meta_cap'], 10, 4);
			add_filter('posts_clauses',	[$this, 'filter_clauses'], 10, 2);

			if(is_object_in_taxonomy($post_type, 'category')){
				add_filter('disable_categories_dropdown', '__return_true');
			}

			$row_actions_filter	= is_post_type_hierarchical($post_type) ? 'page_row_actions' : 'post_row_actions';

			add_filter($row_actions_filter,	[$this, 'filter_row_actions'], 1, 2);

			add_filter('manage_'.$post_type.'_posts_columns',		[$this, 'filter_columns']);
			add_action('manage_'.$post_type.'_posts_custom_column',	[$this, 'filter_custom_column'], 10, 2);

			add_filter('post_column_taxonomy_links',	[$this, 'filter_taxonomy_links'], 10, 3);
		}

		add_filter('manage_'.$screen_id.'_sortable_columns',	[$this, 'filter_sortable_columns']);
	}

	protected function get_nonce_action($key, $id=0){
		$nonce_action	= $key.'-'.$this->post_type.'-posts';

		return $id ? $nonce_action.'-'.$id : $nonce_action;
	}

	protected function get_items($ids){
		return WPJAM_Post::get_by_ids($ids);
	}

	protected function get_item($id){
		return WPJAM_Post::get($id);
	}

	protected function filter_fileds($fields, $key, $id){
		$fields	= apply_filters_deprecated('wpjam_'.$this->post_type.'_posts_fields', [$fields, $key, $id, $this->post_type], 'WPJAM Basic 4.6');

		if($key && $id && !is_array($id)){
			$fields	= array_merge(['title'=>['title'=>$this->_args['title'].'标题', 'type'=>'view', 'value'=>get_post($id)->post_title]], $fields);
		}

		return $fields;
	}

	protected function filter_list_action_result($result, $list_action, $id, $data){
		$hook	= 'wpjam_'.$this->post_type.'_posts_list_action';
		return apply_filters_deprecated($hook, [$result, $list_action, $id, $data, $this->post_type], 'WPJAM Basic 4.6');
	}

	public function single_row($raw_item){
		global $post, $authordata;

		if(is_numeric($raw_item)){
			$post	= get_post($raw_item);
		}else{
			$post	= $raw_item;
		}

		$authordata = get_userdata($post->post_author);
		$post_type	= $post->post_type;

		if($post_type == 'attachment'){
			$wp_list_table = _get_list_table('WP_Media_List_Table', ['screen'=>get_current_screen()]);

			$post_owner = ( get_current_user_id() == $post->post_author ) ? 'self' : 'other';
			?>
			<tr id="post-<?php echo $post->ID; ?>" class="<?php echo trim( ' author-' . $post_owner . ' status-' . $post->post_status ); ?>">
				<?php $wp_list_table->single_row_columns($post); ?>
			</tr>
			<?php
		}else{
			$wp_list_table = _get_list_table('WP_Posts_List_Table', ['screen'=>get_current_screen()]);
			$wp_list_table->single_row($post);
		}
	}

	public function value_callback($name, $id){
		if($id && metadata_exists('post', $id, $name)){
			return get_post_meta($id, $name, true);
		}

		return null;
	}

	public function add_taxonomy_dropdown($post_type){
		foreach(get_object_taxonomies($post_type, 'objects') as $taxonomy => $tax_obj){
			if(empty($tax_obj->show_admin_column)){
				continue;
			}

			$filterable	= $tax_obj->filterable;

			if($taxonomy == 'category' && is_null($filterable)){
				$filterable	= true;
			}

			if(empty($filterable)){
				return;
			}

			$query_var	= $tax_obj->query_var;
			$query_key	= wpjam_get_taxonomy_query_key($taxonomy);
			$selected	= '';

			if(!empty($_REQUEST[$query_key])){
				$selected	= $_REQUEST[$query_key];
			}elseif(!empty($query_var) && !empty($_REQUEST[$query_var])){
				if($term	= get_term_by('slug', $_REQUEST[$query_var], $taxonomy)){
					$selected	= $term->term_id;
				}
			}elseif(!empty($_REQUEST['taxonomy']) && ($_REQUEST['taxonomy'] == $taxonomy) && !empty($_REQUEST['term'])){
				if($term	= get_term_by('slug', $_REQUEST['term'], $taxonomy)){
					$selected	= $term->term_id;
				}
			}

			if($tax_obj->hierarchical){
				wp_dropdown_categories([
					'taxonomy'			=> $taxonomy,
					'show_option_all'	=> $tax_obj->labels->all_items,
					'show_option_none'	=> '没有设置',
					'name'				=> $query_key,
					'selected'			=> (int)$selected,
					'hierarchical'		=> true
				]);
			}else{
				echo wpjam_get_field_html([
					'key'			=> $query_key,
					'value'			=> $selected,
					'type'			=> 'text',
					'data_type'		=> 'taxonomy',
					'taxonomy'		=> $taxonomy,
					'placeholder'	=> '请输入'.$tax_obj->label,
					'title'			=> '',
					'class'			=> ''
				]);
			}
		}
	}

	public function filter_request($query_vars){
		$tax_query	= [];

		foreach (get_object_taxonomies($this->post_type, 'objects') as $taxonomy=>$tax_obj){
			if(!$tax_obj->show_ui){
				continue;
			}

			$tax	= $taxonomy == 'post_tag' ? 'tag' : $taxonomy;

			if($tax != 'category'){
				if(!empty($_REQUEST[$tax.'_id'])){
					$query_vars[$tax.'_id']	= (int)$_REQUEST[$tax.'_id'];
				}
			}

			if(!empty($_REQUEST[$tax.'__and'])){
				$tax__and	= wp_parse_id_list($_REQUEST[$tax.'__and']);

				if(count($tax__and) == 1){
					if (!isset($_REQUEST[$tax.'__in'])){
						$_REQUEST[$tax.'__in']	= [];
					}

					$_REQUEST[$tax.'__in'][]	= absint(reset($tax__and));
				}else{
					$tax__and		= array_map('absint', array_unique($tax__and));
					$tax_query[]	= [
						'taxonomy'	=> $taxonomy,
						'terms'		=> $tax__and,
						'field'		=> 'term_id',
						'operator'	=> 'AND',
						// 'include_children'	=> false,
					];
				}
			}

			if(!empty($_REQUEST[$tax.'__in'])){
				$tax__in		= wp_parse_id_list($_REQUEST[$tax.'__in']);
				$tax__in		= array_map('absint', array_unique($tax__in));

				$tax_query[]	= [
					'taxonomy'	=> $taxonomy,
					'terms'		=> $tax__in,
					'field'		=> 'term_id'
				];
			}

			if(!empty($_REQUEST[$tax.'__not_in'])){
				$tax__not_in	= wp_parse_id_list($_REQUEST[$tax.'__not_in']);
				$tax__not_in	= array_map('absint', array_unique($tax__not_in));

				$tax_query[]	= [
					'taxonomy'	=> $taxonomy,
					'terms'		=> $tax__not_in,
					'field'		=> 'term_id',
					'operator'	=> 'NOT IN'
				];
			}
		}

		if($tax_query){
			$tax_query['relation']		= $_REQUEST['tax_query_relation'] ?? 'and'; 
			$query_vars['tax_query']	= $tax_query;
		}

		return $query_vars;
	}

	public function filter_row_actions($row_actions, $post){
		if($post->post_status == 'trash'){
			$row_actions['post_id'] = 'ID: '.$post->ID;
			return $row_actions;
		}

		if($this->_args['actions']){
			$actions	= [];

			foreach($this->_args['actions'] as $key => $action){
				if(isset($action['post_status'])){
					$post_statuses	= is_array($action['post_status']) ? $action['post_status'] : [$action['post_status']];

					if(!in_array($post->post_status, $post_statuses)){
						continue;
					}
				}

				$actions[$key]	= $action;
			}

			if($actions){
				$row_actions	= array_merge($row_actions, $this->get_row_actions($actions, $post->ID, $post));
			}
		}

		if(isset($row_actions['trash'])){
			$trash	= wpjam_array_pull($row_actions, 'trash');

			$row_actions['trash']	= $trash;
		}

		$row_actions['post_id'] = 'ID: '.$post->ID;

		return $row_actions;
	}

	public function filter_bulk_actions($bulk_actions=[]){
		return array_merge($bulk_actions, $this->_args['bulk_actions']);
	}

	public function filter_map_meta_cap($caps, $cap, $user_id, $args){
		if($cap == 'edit_post'){
			if(empty($args[0])){
				$pt_obj	= get_post_type_object($this->post_type);
				return $pt_obj->map_meta_cap ? [$pt_obj->cap->edit_posts] : [$pt_obj->cap->$cap];
			}
		}

		return $caps;
	}

	public function filter_columns($columns){
		if($this->_args['columns']){
			wpjam_array_push($columns, $this->_args['columns'], 'date'); 
		}

		return $columns;
	}

	public function filter_custom_column($column_name, $post_id){
		if(metadata_exists('post', $post_id, $column_name)){
			$column_value	= get_post_meta($post_id, $column_name, true);
		}else{
			$column_value	= null;
		}

		echo $this->column_callback($column_value, $column_name, $post_id) ?? '';
	}

	public function filter_sortable_columns($columns){
		return array_merge($columns, $this->_args['sortable_columns']);
	}

	public function filter_clauses($clauses, $wp_query){
		if($wp_query->is_main_query() && $wp_query->is_search()){
			global $wpdb;

			$search_term	= $wp_query->query['s'];

			if(is_numeric($search_term)){
				$clauses['where'] = str_replace('('.$wpdb->posts.'.post_title LIKE', '('.$wpdb->posts.'.ID = '.$search_term.') OR ('.$wpdb->posts.'.post_title LIKE', $clauses['where']);
			}elseif(preg_match("/^(\d+)(,\s*\d+)*\$/", $search_term)){
				$clauses['where'] = str_replace('('.$wpdb->posts.'.post_title LIKE', '('.$wpdb->posts.'.ID in ('.$search_term.')) OR ('.$wpdb->posts.'.post_title LIKE', $clauses['where']);
			}

			if($search_metas = $wp_query->get('search_metas')){
				$clauses['where']	= preg_replace_callback('/\('.$wpdb->posts.'.post_title LIKE (.*?)\) OR/', function($matches) use($search_metas){
					global $wpdb;
					$search_metas	= "'".implode("', '", $search_metas)."'";

					return "EXISTS (SELECT * FROM {$wpdb->postmeta} WHERE {$wpdb->postmeta}.post_id={$wpdb->posts}.ID AND meta_key IN ({$search_metas}) AND meta_value LIKE ".$matches[1].") OR ".$matches[0];
				}, $clauses['where']);
			}
		}

		return $clauses;
	}

	public function filter_taxonomy_links($term_links, $taxonomy, $terms){
		$permastruct	= wpjam_get_permastruct($taxonomy);

		if(empty($permastruct) || strpos($permastruct, '/%'.$taxonomy.'_id%')){
			$query_var	= get_taxonomy($taxonomy)->query_var;
			$query_key	= wpjam_get_taxonomy_query_key($taxonomy);

			foreach($terms as $i => $t){
				$query_str		= $query_var ? $query_var.'='.$t->slug : 'taxonomy='.$taxonomy.'&#038;term='.$t->slug;
				$term_links[$i]	= str_replace($query_str, $query_key.'='.$t->term_id, $term_links[$i]);
			}
		}

		return $term_links;
	}

	public function add_thumbnail_wrap($html){
		return preg_replace_callback('/<a class="row-title" href=".*?post=(\d+).*?"/is', function($matches){
			$post_id	= $matches[1];
			$thumbnail	= get_the_post_thumbnail($post_id, [50,50]) ?: '<span class="no-thumbnail">暂无图片</span>';

			if(post_type_supports($this->post_type, 'thumbnail') && current_user_can('edit_post', $post_id)){
				$thumbnail = wpjam_get_list_table_row_action('set_thumbnail',['id'=>$post_id, 'title'=>$thumbnail]);
			}

			return $thumbnail.$matches[0]; 
		}, $html);
	}

	public function filter_ajax_response($response){
		if($this->get_action('set_thumbnail') && isset($response['data'])){
			if(is_array($response['data'])){
				$response['data']	= array_map([$this, 'add_thumbnail_wrap'], $response['data']);
			}else{
				$response['data']	= $this->add_thumbnail_wrap($response['data']);
			}
		}

		return $response;
	}

	public function filter_html($html){
		if(!wp_doing_ajax() && $this->get_action('add')){
			$html	= preg_replace('/<a href=".*?" class="page-title-action">.*?<\/a>/i', $this->get_row_action('add', ['class'=>'page-title-action']), $html);
		}

		if($this->get_action('set_thumbnail')){
			$html	= $this->add_thumbnail_wrap($html);
		}

		return $html;
	}

	public function pre_get_posts($wp_query){
		if($sortable_columns = $this->_args['sortable_columns']){
			$orderby	= $wp_query->get('orderby');

			if($orderby && is_string($orderby) && isset($sortable_columns[$orderby])){
				$fields	= $this->get_fields();
				$field	= $fields[$orderby] ?? '';

				$orderby_type = $field['sortable_column'] ?? 'meta_value';

				if(in_array($orderby_type, ['meta_value_num', 'meta_value'])){
					$wp_query->set('meta_key', $orderby);
					$wp_query->set('orderby', $orderby_type);
				}else{
					$wp_query->set('orderby', $orderby);
				}
			}
		}
	}

	public function admin_head(){
		if($bulk_actions = $this->_args['bulk_actions']){
		?>

		<script type="text/javascript">
		jQuery(function($){
			<?php 
			foreach($bulk_actions as $action_key => $bulk_action) { 
				$bulk_action	= $this->_args['actions'][$action_key];

				$datas	= ['action'=>$action_key, 'bulk'=>true];

				$datas['page_title']	= $bulk_action['page_title']??$bulk_action['title']; 
				$datas['nonce']			= $this->create_nonce('bulk_'.$action_key); 

				if(!empty($bulk_action['direct'])){
					$datas['direct']	= true;
				}

				if(!empty($bulk_action['confirm'])){
					$datas['confirm']	= true;
				}

				echo '$(\'.bulkactions option[value='.$action_key.']\').data('.wpjam_json_encode($datas).')'."\n";
			} 
			?>
		});
		</script>

		<?php }
	}
}

class WPJAM_Terms_List_Table extends WPJAM_List_Table{
	private $taxonomy	= '';

	public function __construct($args = []){
		$current_screen	= get_current_screen();

		$screen_id	= $current_screen->id;
		$taxonomy	= $current_screen->taxonomy;
		$tax_obj	= get_taxonomy($taxonomy);

		$this->taxonomy		= $taxonomy;
		$args['title']		= $tax_obj->label;
		$args['capability']	= $tax_obj->cap->edit_terms;
		$args['data_type']	= 'term_meta';
		$args['actions']	= wpjam_sort_items(WPJAM_List_Table_Action::get_by_screen_id($screen_id));
		$args['actions']	= apply_filters_deprecated('wpjam_'.$taxonomy.'_terms_actions', [$args['actions'], $taxonomy], 'WPJAM Basic 4.6');

		$this->_args	= $this->parse_args($args);

		if(wp_doing_ajax()){
			add_action('wp_ajax_wpjam-list-table-action', [$this, 'ajax_response']);
		}else{
			add_action('admin_head',	[$this, 'admin_head']);
			add_action('admin_footer',	[$this, '_js_vars']);
		}
		
		if(!wp_doing_ajax() || (wp_doing_ajax() && in_array($_POST['action'], ['inline-save-tax', 'add-tag']))){
			add_action('wpjam_html',	[$this, 'filter_html']);
		}

		if(wp_doing_ajax() && $_POST['action'] == 'wpjam-list-table-action' && $_POST['list_action_type'] != 'form'){
			add_action('wpjam_ajax_response',	[$this, 'filter_ajax_response']);
		}

		add_action('parse_term_query',	[$this, 'parse_term_query']);

		add_filter('bulk_actions'.$screen_id,	[$this, 'filter_bulk_actions']);
		add_filter($taxonomy.'_row_actions',	[$this, 'filter_row_actions'],1,2);

		add_filter('manage_'.$screen_id.'_columns',				[$this, 'filter_columns']);
		add_filter('manage_'.$taxonomy.'_custom_column',		[$this, 'filter_custom_column'],10,3);
		add_filter('manage_'.$screen_id.'_sortable_columns',	[$this, 'filter_sortable_columns']);
	}

	protected function get_nonce_action($key, $id=0){
		$nonce_action	= $key.'-'.$this->taxonomy.'-terms';

		return $id ? $nonce_action.'-'.$id : $nonce_action;
	}

	protected function get_items($ids){
		return WPJAM_Term::get_by_ids($ids);
	}

	protected function get_item($id){
		return WPJAM_Term::get($id);
	}

	protected function filter_fileds($fields, $key, $id){
		$fields		= apply_filters_deprecated('wpjam_'.$this->taxonomy.'_terms_fields', [$fields, $key, $id, $this->taxonomy], 'WPJAM Basic 4.6');

		if($key && $id && !is_array($id)){
			$fields	= array_merge(['title'=>['title'=>$this->_args['title'], 'type'=>'view', 'value'=>get_term($id)->name]], $fields);
		}

		return $fields;
	}

	protected function filter_list_action_result($result, $list_action, $id, $data){
		$hook	= 'wpjam_'.$this->taxonomy.'_terms_list_action';
		return apply_filters_deprecated($hook, [$result, $list_action, $id, $data, $this->taxonomy], 'WPJAM Basic 4.6');
	}

	public function single_row($raw_item){
		if(is_numeric($raw_item)){
			$term	= get_term($raw_item);
		}else{
			$term	= $raw_item;
		}

		$level	= $term->parent ? count(get_ancestors($term->term_id, get_current_screen()->taxonomy, 'taxonomy')) : 0;

		$wp_list_table = _get_list_table('WP_Terms_List_Table', ['screen'=>get_current_screen()]);
		$wp_list_table->single_row($term, $level);
	}

	public function value_callback($name, $id){
		if($id && metadata_exists('term', $id, $name)){
			return get_term_meta($id, $name, true);
		}

		return null;
	}

	public function filter_row_actions($row_actions, $term){
		if($this->_args['actions']){
			$actions	= [];

			foreach($this->_args['actions'] as $key => $action){
				if(isset($action['parent'])){
					if($term->parent != $action['parent']){
						continue;
					}
				}

				$actions[$key]	= $action;
			}

			if($actions){
				$row_actions	= array_merge($row_actions, $this->get_row_actions($actions, $term->term_id, $term));
			}
		}

		$tax_obj	= get_taxonomy($term->taxonomy);
		$supports	= $tax_obj->supports;

		if(!in_array('slug', $supports)){
			unset($row_actions['inline hide-if-no-js']);
		}

		$row_actions['term_id'] = 'ID：'.$term->term_id;

		return $row_actions;
	}

	public function filter_columns($columns){
		$tax_obj	= get_taxonomy($this->taxonomy);
		$supports	= $tax_obj->supports;

		if(!in_array('slug', $supports)){
			unset($columns['slug']);
		}

		if(!in_array('description', $supports)){
			unset($columns['description']);
		}

		if($this->_args['columns']){
			wpjam_array_push($columns, $this->_args['columns'], 'posts'); 
		}

		return $columns;
	}

	public function filter_custom_column($value, $column_name, $term_id){
		if(metadata_exists('term', $term_id, $column_name)){
			$column_value = get_term_meta($term_id, $column_name, true);
		}else{
			$column_value = null;
		}

		return $this->column_callback($column_value, $column_name, $term_id) ?? $value;
	}

	public function filter_sortable_columns($columns){
		return array_merge($columns, $this->_args['sortable_columns']);
	}

	public function parse_term_query($term_query){
		if($sortable_columns	= $this->_args['sortable_columns']){
			$orderby	= $term_query->query_vars['orderby'];

			if($orderby && isset($sortable_columns[$orderby])){

				$fields	= $this->get_fields();
				$field	= $fields[$orderby] ?? '';

				$orderby_type = ($field['sortable_column'] == 'meta_value_num')?'meta_value_num':'meta_value';

				$term_query->query_vars['meta_key']	= $orderby;
				$term_query->query_vars['orderby']	= $orderby_type;
			}
		}
	}

	public function filter_bulk_actions($bulk_actions=[]){
		return array_merge($bulk_actions, $this->_args['bulk_actions']);
	}

	public function admin_head(){
		if($bulk_actions = $this->_args['bulk_actions']){ $actions = $this->_args['actions']; ?>

		<script type="text/javascript">
		jQuery(function($){
			<?php foreach($bulk_actions as $action_key => $bulk_action) { 
				$bulk_action	= $actions[$action_key];

				$datas	= ['action'=>$action_key, 'bulk'=>true];

				$datas['page_title']	= $bulk_action['page_title']??$bulk_action['title']; 
				$datas['nonce']			= $this->create_nonce('bulk_'.$action_key); 

				if(!empty($bulk_action['direct'])){
					$datas['direct']	= true;
				}

				if(!empty($bulk_action['confirm'])){
					$datas['confirm']	= true;
				}

				echo '$(\'.bulkactions option[value='.$action_key.']\').data('.wpjam_json_encode($datas).')'."\n";
			}?>
		});
		</script>

		<?php } 
	}

	public function replace_edit_link($html){
		return preg_replace_callback('/<tr id="tag-(\d+)" class=".*?">.*?<\/tr>/is', function($matches){
			$term_id	= $matches[1];
			$query_var	= get_taxonomy($this->taxonomy)->query_var;
			$query_key	= wpjam_get_taxonomy_query_key($this->taxonomy);
			$search		= $query_var ? '?'.$query_var.'='.get_term($term_id)->slug : '?taxonomy='.$this->taxonomy.'&#038;term='.get_term($term_id)->slug;
			$replace	= '?'.$query_key.'='.$term_id;

			return str_replace($search, $replace, $matches[0]);
		}, $html);
	}

	public function add_thumbnail_wrap($html){
		return preg_replace_callback('/<strong><a class="row-title" href=".*?tag_ID=(\d+).*?"/is', function($matches){
			$term_id	= $matches[1];
			$thumb_url	= wpjam_get_term_thumbnail_url($term_id, [100, 100]);
			$thumbnail	= $thumb_url ? '<img class="wp-term-image" src="'.$thumb_url.'"'.image_hwstring(50,50).' />' : '<span class="no-thumbnail">暂无图片</span>';
			$taxonomy	= get_term($term_id)->taxonomy;
			$capability	= get_taxonomy($taxonomy)->cap->edit_terms;

			if(current_user_can($capability)){
				$thumbnail = wpjam_get_list_table_row_action('set_thumbnail', ['id'=>$term_id, 'title'=>$thumbnail]);
			}

			return $thumbnail.$matches[0];
		}, $html);
	}

	public function filter_html($html){
		$permastruct	= wpjam_get_permastruct($this->taxonomy);

		if(empty($permastruct) || strpos($permastruct, '/%'.$this->taxonomy.'_id%')){
			$html	= $this->replace_edit_link($html);
		}

		if($this->get_action('set_thumbnail')){
			$html	= $this->add_thumbnail_wrap($html);
		}

		return $html;
	}

	public function filter_ajax_response($response){
		if(isset($response['data'])){
			$permastruct	= wpjam_get_permastruct($this->taxonomy);

			if(empty($permastruct) || strpos($permastruct, '/%'.$this->taxonomy.'_id%')){
				if(is_array($response['data'])){
					$response['data']	= array_map([$this, 'replace_edit_link'], $response['data']);
				}else{
					$response['data']	= $this->replace_edit_link($response['data']);
				}
			}

			if($this->get_action('set_thumbnail')){
				if(is_array($response['data'])){
					$response['data']	= array_map([$this, 'add_thumbnail_wrap'], $response['data']);
				}else{
					$response['data']	= $this->add_thumbnail_wrap($response['data']);
				}
			}	
		}

		return $response;
	}
}

class WPJAM_Users_List_Table extends WPJAM_List_Table{
	public function __construct($args = []){
		$current_screen	= get_current_screen();

		$screen_id	= $current_screen->id;

		$args['data_type']	= 'user_meta';
		$args['title']		= '用户';
		$args['capability']	= 'edit_user';
		$args['data_type']	= 'user_meta';
		$args['actions']	= wpjam_sort_items(WPJAM_List_Table_Action::get_by_screen_id($screen_id));
		$args['actions']	= apply_filters_deprecated('wpjam_users_actions', [$args['actions']], 'WPJAM Basic 4.6');

		$this->_args	= $this->parse_args($args);

		if(wp_doing_ajax()){
			add_action('wp_ajax_wpjam-list-table-action', [$this, 'ajax_response']);
		}else{
			add_action('admin_footer',	[$this, '_js_vars']);
		}

		add_filter('user_row_actions',	[$this, 'filter_row_actions'], 1, 2);

		add_filter('manage_users_columns',			[$this, 'filter_columns']);
		add_filter('manage_users_custom_column',	[$this, 'filter_custom_column'],10,3);
		add_filter('manage_users_sortable_columns',	[$this, 'filter_sortable_columns']);
	}

	protected function get_nonce_action($key, $id=0){
		$nonce_action	= $key.'-users';

		return $id ? $nonce_action.'-'.$id : $nonce_action;
	}

	protected function get_items($ids){
		// return WPJAM_User::get_by_ids($ids);
	}

	protected function get_item($id){
		// return WPJAM_User::get($id);
	}

	protected function filter_fileds($fields, $key, $id){
		$fields		= apply_filters_deprecated('wpjam_users_fields', [$fields, $key, $id], 'WPJAM Basic 4.6');

		if($key && $id && !is_array($id)){
			$fields	= array_merge(['name'=>['title'=>'用户', 'type'=>'view', 'value'=>get_userdata($id)->display_name]], $fields);
		}

		return $fields;
	}

	protected function filter_list_action_result($result, $list_action, $id, $data){
		$hook	= 'wpjam_users_list_action';
		return apply_filters_deprecated($hook, [$result, $list_action, $id, $data, 'user'], 'WPJAM Basic 4.6');
	}

	public function single_row($raw_item){
		$wp_list_table = _get_list_table('WP_Users_List_Table', ['screen'=>get_current_screen()]);

		echo $wp_list_table->single_row($raw_item);
	}

	public function value_callback($name, $id){
		if($id && metadata_exists('user', $id, $name)){
			return get_user_meta($id, $name, true);
		}

		return null;
	}

	public function filter_row_actions($row_actions, $user){
		if($this->_args['actions']){
			$actions	= [];

			foreach($this->_args['actions'] as $key => $action){
				if(isset($action['roles'])){
					if(!array_intersect($item->roles, $action['roles'])){
						continue;
					}
				}

				$actions[$key]	= $action;
			}

			if($actions){
				$row_actions	= array_merge($row_actions, $this->get_row_actions($actions, $user->ID, $user));
			}
		}

		$row_actions['user_id'] = 'ID: '.$user->ID;

		return $row_actions;
	}

	public function filter_columns($columns){
		if($this->_args['columns']){
			wpjam_array_push($columns, $this->_args['columns'], 'posts'); 
		}

		return $columns;
	}

	public function filter_custom_column($value, $column_name, $user_id){
		if(metadata_exists('user', $user_id, $column_name)){
			$column_value = get_user_meta($user_id, $column_name, true);
		}else{
			$column_value = null;
		}

		return $this->column_callback($column_value, $column_name, $user_id) ?? $value;
	}

	public function filter_sortable_columns($columns){
		return array_merge($columns, $this->_args['sortable_columns']);
	}
}